#include "stdafx.h"
#include "BezierHelper.h"

double BezierCurve::Ni(int n, int i)
{
	double ni;
	double a1 = FactorialLookup[n];
	double a2 = FactorialLookup[i];
	double a3 = FactorialLookup[n - i];
	ni = a1 / (a2 * a3);
	return ni;
}

double BezierCurve::Bernstein(int n, int i, double t)
{
	double basis;
	double ti; /* t^i */
	double tni; /* (1 - t)^i */

				/* Prevent problems with pow */

	if (t == 0.0 && i == 0)
		ti = 1.0;
	else
		ti = pow(t, i);

	if (n == i && t == 1.0)
		tni = 1.0;
	else
		tni = pow((1 - t), (n - i));

	//Bernstein basis
	basis = Ni(n, i) * ti * tni;
	return basis;
}


BezierCurve::BezierCurve()
{
	FactorialLookup.resize(33);
	FactorialLookup[0] = 1.0;
	FactorialLookup[1] = 1.0;
	FactorialLookup[2] = 2.0;
	FactorialLookup[3] = 6.0;
	FactorialLookup[4] = 24.0;
	FactorialLookup[5] = 120.0;
	FactorialLookup[6] = 720.0;
	FactorialLookup[7] = 5040.0;
	FactorialLookup[8] = 40320.0;
	FactorialLookup[9] = 362880.0;
	FactorialLookup[10] = 3628800.0;
	FactorialLookup[11] = 39916800.0;
	FactorialLookup[12] = 479001600.0;
	FactorialLookup[13] = 6227020800.0;
	FactorialLookup[14] = 87178291200.0;
	FactorialLookup[15] = 1307674368000.0;
	FactorialLookup[16] = 20922789888000.0;
	FactorialLookup[17] = 355687428096000.0;
	FactorialLookup[18] = 6402373705728000.0;
	FactorialLookup[19] = 121645100408832000.0;
	FactorialLookup[20] = 2432902008176640000.0;
	FactorialLookup[21] = 51090942171709440000.0;
	FactorialLookup[22] = 1124000727777607680000.0;
	FactorialLookup[23] = 25852016738884976640000.0;
	FactorialLookup[24] = 620448401733239439360000.0;
	FactorialLookup[25] = 15511210043330985984000000.0;
	FactorialLookup[26] = 403291461126605635584000000.0;
	FactorialLookup[27] = 10888869450418352160768000000.0;
	FactorialLookup[28] = 304888344611713860501504000000.0;
	FactorialLookup[29] = 8841761993739701954543616000000.0;
	FactorialLookup[30] = 265252859812191058636308480000000.0;
	FactorialLookup[31] = 8222838654177922817725562880000000.0;
	FactorialLookup[32] = 263130836933693530167218012160000000.0;
}
double BezierCurve::factorial(int n)
{
	if (n >= 0 && n <= 32) {
		return FactorialLookup[n];
	}
	else {
		return 0;
	}
}
vector<Point2f> BezierCurve::Bezier2D(Point2f a, Point2f b, Point2f c, Point2f d)
{
	if (a.x == b.x && a.y == b.y && c.x == d.x && c.y == d.y)
	{
		vector<Point2f> rtn;
		Point2f pace = d - a;
		pace.x /= 99.0f;
		pace.y /= 99.0f;
		for (int i = 0; i < 100; i++)
		{
			Point2f p((float)i * pace.x, (float)i * pace.y);
			rtn.push_back(a + p);
		}
		return rtn;
	}
	else
	{
		vector<double> ptind;
		ptind.push_back(a.x);
		ptind.push_back(a.y);
		ptind.push_back(b.x);
		ptind.push_back(b.y);
		ptind.push_back(c.x);
		ptind.push_back(c.y);
		ptind.push_back(d.x);
		ptind.push_back(d.y);

		vector<double> p = Bezier2D(ptind, 100);
		vector<Point2f> rtn(100);
		for (int i = 0; i < 100; i++)
		{
#ifdef TEST_ALT
			rtn[i] = Point2f((int)p[2 * i], (int)p[2 * i + 1]);
#else
			rtn[i] = Point2f((float)p[2 * i], (float)p[2 * i + 1]);
#endif
		}
		return rtn;
	}
}
vector<double> BezierCurve::Bezier2D(vector<double> b, int cpts)
{
	int npts = b.size() / 2;
	int icount, jcount;
	double step, t;
	vector<double> p(200);
	// Calculate points on curve

	icount = 0;
	t = 0;
	step = (double)1.0 / (cpts - 1);

	for (int i1 = 0; i1 != cpts; i1++)
	{
		if ((1.0 - t) < 5e-6)
			t = 1.0;

		jcount = 0;
		p[icount] = 0.0;
		p[icount + 1] = 0.0;
		for (int i = 0; i != npts; i++)
		{
			double basis = Bernstein(npts - 1, i, t);
			p[icount] += basis * b[jcount];
			p[icount + 1] += basis * b[jcount + 1];
			jcount = jcount + 2;
		}

		icount += 2;
		t += step;
	}
	return p;
}